# Parameter Description 

## WorkParaBean Invoking parameter configuration instruction
### setFlowId
| Category              | Description               |
|-----------------------|---------------------------|
| Parameter format      | Character string          |
| Parameter implication | Process ID or instance ID |
| Scope of action       | Create instance           |
| Required              | Required                  |

### setOuterConfig
| Category              | Description                                                                                                                                                                                                           |
|-----------------------|-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| Parameter format      | Character string                                                                                                                                                                                                      |
| Parameter implication | External setting ID name                                                                                                                                                                                              |
| Scope of action       | Create instance                                                                                                                                                                                                       |
| Required              | No                                                                                                                                                                                                                    |
| Other description     | To obtain the operator from external settings, it is necessary to set an external operator at the node operator in the process definition<br>![Process operation class diagram](../assets/img/api/setOuterConfig.png) |

### setWorkId
| Category              | Description                      |
|-----------------------|----------------------------------|
| Parameter format      | Character string                 |
| Parameter implication | Instance ID                      |
| Scope of action       | Open instance、Instance operation |
| Required              | Required                         |
### setTrackId
| Category   | Description                                                                           |
|------|-----------------------------------------------------------------------------|
| Parameter format | Character string                                                                        |
| Parameter implication | Current path ID                                                                      |
| Scope of action | Open instance、Instance operation                                                                   |
| Required | Required during instance operation                                                                     |
| Other description | When opening an instance，If there are multiple active paths, one must be passed in, otherwise the system will prompt to select the path；<br>When performing an Instance operation, a path must be passed in. When there is only one active path, it can be obtained from the opened instance object |
### setNodeId
| Category              | Description                                                                               |
|-----------------------|-------------------------------------------------------------------------------------------|
| Parameter format      | Character string                                                                          |
| Parameter implication | Current node ID                                                                           |
| Scope of action       | Instance operation                                                                        |
| Required              | Required                                                                                  |
| Other description     | When performing instance operations, the current node ID must be passed in as a parameter |
### setTenantCode
| Category              | Description                      |
|-----------------------|----------------------------------|
| Parameter format      | Character string                 |
| Parameter implication | Current tenant ID                |
| Scope of action       | Open instance、Instance operation |
| Required              | Required                         |
| Other description     | Obtained based on session        |
### setFlowIdentifier
| Category              | Description                                                                                                                                                                                                                                                                               |
|-----------------------|-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| Parameter format      | Character string                                                                                                                                                                                                                                                                          |
| Parameter implication | Process Data Source ID                                                                                                                                                                                                                                                                    |
| Scope of action       | Open instance、Instance operation                                                                                                                                                                                                                                                          |
| Required              | Required                                                                                                                                                                                                                                                                                  |
| Other description     | If the process data source is not passed in, the system will automatically obtain it from the interface method based on tenant Code<br>com.horizon.third.ThirdAdapterFactory.getIdentifierFromTenantid(String tenant)<br>It is recommended to obtain and pass in from the current session |
### setDataIdentifier
| Category              | Description                                                                                        |
|-----------------------|----------------------------------------------------------------------------------------------------|
| Parameter format      | Character string                                                                                   |
| Parameter implication | Business data surce identifier                                                                     |
| Scope of action       | Open instance、Instance operation                                                                   |
| Required              | Required                                                                                           |
| Other description     | If this data source is inconsistent with flowIdentifier, it cannot be processed in one transaction |
### setUserId
| Category              | Description                                                                              |
|-----------------------|------------------------------------------------------------------------------------------|
| Parameter format      | Character string<br> 1.U_userid  2.userid  3.U_userid/X_XXXid                            |
| Parameter implication | Current user ID                                                                          |
| Scope of action       | Open instance、Instance operation                                                         |
| Required              | Required                                                                                 |
| Other description     | If setSubject is used for assignment, then the X_ XXXid in this method  will be replaced |
### setSubjection
| Category              | Description                                                                             |
|-----------------------|-----------------------------------------------------------------------------------------|
| Parameter format      | Character string<br> 1.U_userid  2.userid  3.U_userid/X_XXXid                           |
| Parameter implication | Current user ID                                                                         |
| Scope of action       | Open instance、Instance operation                                                        |
| Required              | Required                                                                                |
| Other description     | If setSubject is used for assignment, then the X_ XXXid in this method will be replaced |
### setAllUserids
| Category              | Description                                                                                                                                                                                                                                                                  |
|-----------------------|------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| Parameter format      | Character string                                                                                                                                                                                                                                                             |
| Parameter implication | All permission sets for the current user                                                                                                                                                                                                                                     |
| Scope of action       | Open instance                                                                                                                                                                                                                                                                |
| Required              | No                                                                                                                                                                                                                                                                           |
| Other description     | If the permission set of the current user is not passed in, the system will automatically call the interface method to obtain it (if the interface method has a corresponding implementation)<br>ThirdAdapterFactory.getOrganizationInstance(tenantid).getAllUserids(userid) |
### setUserName
| Category              | Description                      |
|-----------------------|----------------------------------|
| Parameter format      | Character string                 |
| Parameter implication | Current username                 |
| Scope of action       | Open instance、Instance operation |
| Required              | No                               |
### setDeptName
| Category              | Description                      |
|-----------------------|----------------------------------|
| Parameter format      | Character string                 |
| Parameter implication | Current user department name     |
| Scope of action       | Open instance、Instance operation |
| Required              | No                               |
| Other description     | Used for logging                 |

### setParentWorkid
| Category              | Description                                                                                                          |
|-----------------------|----------------------------------------------------------------------------------------------------------------------|
| Parameter format      | Character string                                                                                                     |
| Parameter implication | The workid of the parent instance                                                                                    |
| Scope of action       | Open instance                                                                                                        |
| Required              | No                                                                                                                   |
| Other description     | Set whether the current instance needs to inherit data from other instances, usually used when starting a subprocess |
### setNextNodeId
| Category              | Description                                                                                                                                                                                                                                                                                                                                                                                                                                  |
|-----------------------|----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| Parameter format      | Character string set next node Id of  process instance <br>format:<br>Submitting operation format: Routing Id~Node Id, multiple values separated by '&'<br>Except for the submitting operation, all other formats are node IDs, with multiple values separated by '&'<br>For example, submit:<br>Line1~Node1  Single valued, Line1~Node1&Line2~Node2 ,Multiple values ,such as returns:<br>Node1  Single value, Node1&Node2 ,Multiple values |
| Parameter implication | Next Node List                                                                                                                                                                                                                                                                                                                                                                                                                               |
| Scope of action       | Instance operation                                                                                                                                                                                                                                                                                                                                                                                                                           |
| Required              | Required                                                                                                                                                                                                                                                                                                                                                                                                                                     |
| Other description     | When performing an operation, set the next node. When not set, the system will automatically obtain the next node,<br>If the next node is determined, there is no need to select it and it will be processed directly,If the operator needs to be selected, the node ID can obtain format information from the return value<br> （The main consideration is the slightly different format of node ID when skipping nodes)<br>                 |
### setOtherPara
| Category                                                        | Description                                               |
|-----------------------------------------------------------------|-----------------------------------------------------------|
| Parameter format                                                | LinkedHashMap                                             |
| key Parameter Variable Name, value: Value of parameter variable |
| Parameter implication                                           | Set other additional parameters for process instance flow |
| Scope of action                                                 | Process flow                                              |
| Required                                                        | No                                                        |
### setMessageMoulds
| Category              | Description                                                         |
|-----------------------|---------------------------------------------------------------------|
| Parameter format      | LinkedHashMap<br>key:Node Id_ operator type   value: Title of to-do |
| Parameter implication | Set template of process message sending                             |
| Scope of action       | Process flow                                                        |
| Required              | No                                                                  |
### setTitle
| Category              | Description           |
|-----------------------|-----------------------|
| Parameter format      | Character string      |
| Parameter implication | Title of process flow |
| Scope of action       | To-do                 |
| Required              | No                    |
### setActionClass
| Category              | Description                                                              |
|-----------------------|--------------------------------------------------------------------------|
| Parameter format      | Character string                                                         |
| Parameter implication | Operation implementation class full path                                 |
| Scope of action       | Instance operation                                                       |
| Required              | Required                                                                 |
| Other description     | The operation implement interface,com.horizon.wf.core.work.ICustomAction |
### setSelectSubFlowid
| Category              | Description                                                               |
|-----------------------|---------------------------------------------------------------------------|
| Parameter format      | Character string                                                          |
| Parameter implication | Selected sub_processes 'flowid'                                           |
| Scope of action       | Start sub_process                                                         |
| Required              | Required                                                                  |
| Other description     | When starting sub process operation, provide 'flowid' of the sub _process |
### setUrlApp
| Category              | Description                 |
|-----------------------|-----------------------------|
| Parameter format      | Character string            |
| Parameter implication | The URL  address  of  to-do |
| Scope of action       | To do list                  |
| Required              | No                          |
### setSelectAuthorMap
| Category              | Description                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                |
|-----------------------|----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| Parameter format      | HashMap &#60;String,String&#62; <br>Format for selecting operator :<br>key=Routing ID - operator type(Author,reader..)<br>Node id-operator type(Author,reader..)<br>Operator type (set this value when nodes do not involve changes)<br>value=userid(Multiple, separate by semicolons";")<br>Such as：<br>LinkedHashMap &#60;String,String&#62; selectedUsers = new LinkedHashMap&#60;String,String&#62;();<br>selectedUsers.put("Line1-Author","xxxx;yyyyy");<br>selectedUsers.put("Line1-Reader","xxxx;yyyyy");<br>Note: If it is a jump or other operation, it needs to be provided in the following format：Node1-Author |
| Parameter implication | Node operator                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              |
| Scope of action       | Instance operation                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         |
| Required              | No                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         |
| Other description     | Replace map of object with a parameter map<br>The format of the 'Userid' is the same as the parameter in 'setUserId'<br>There are several types of operator‘s format：<br>1.U_userid,or userid<br>2.U_userid~U_agentid<br>3.U_userid~U_agentid/X_XXXid<br>4.U_userid/X_XXXid<br>5./X_XXXid,or /XXXid,or X_XXXid must have a set type prefix, otherwise it will be classified by 'Userid'<br>                                                                                                                                                                                                                                |
### putSelectAuthorMap
| Category              | Description                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 |
|-----------------------|-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| Parameter format      | HashMap &#60;String,String&#62;<br>Format of selecting operator :<br>key=Routing ID - operator type(Author,reader..)<br>Or node id-operator type(Author,reader..)<br>Operator type (set this value when nodes do not involve changes) value=userid(Multiple, separate by semicolons";")<br>Such as：<br>LinkedHashMap&#60;String,String&#62; selectedUsers = new LinkedHashMap &#60;String,String&#62;(); <br>selectedUsers.put("Line1-Author","xxxx;yyyyy"); <br>selectedUsers.put("Line1-Reader","xxxx;yyyyy"); <br>Note: If it is a jump or other operation, it needs to be provided in the following format：Node1-Author |
| Parameter implication | Operator                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    |
| Scope of action       | Instance operation                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| Required              | No                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| Other description     | Assign all values in the parameter Map to the Map of the object. If the same key already exists in the object, overwrite it<br>The format of the 'Userid' is the same as the parameter in 'setUserId'<br>There are several types of operator‘s format：1.U_userid,or userid<br>2.U_userid~U_agentid<br>3.U_userid~U_agentid/X_XXXid<br>4.U_userid/X_XXXid<br>5./X_XXXid,or /XXXid,or  X_XXXid must have a set type prefix, otherwise it will be classified by 'Userid'<br>                                                                                                                                                   |
### putSelectAuthor
| Category              | Description                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           |
|-----------------------|---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| Parameter format      | Format of selecting operator:<br> key=Routing ID - operator type(Author,reader..)<br>Or node id-operator type(Author,reader..)<br>Operator type (set this value when nodes do not involve changes)<br>value=userid(Multiple, separate by semicolons";")Such as：<br>LinkedHashMap &#60;String,String&#62; selectedUsers = new LinkedHashMap&#60;String,String&#62; ; <br>selectedUsers.put("Line1-Author","xxxx;yyyyy");<br>selectedUsers.put("Line1-Reader","xxxx;yyyyy"); <br>Note: If it is a jump or other operation, it needs to be provided in the following format：Node1-Author |
| Parameter implication | Operator                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              |
| Scope of action       | Instance operation                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    |
| Required              | No                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    |
| Other description     | Save the specified key and value in the Map of the object<br>The format of the 'Userid' is the same as the parameter in 'setUserId'<br>There are several types of operator‘s format：<br>1.U_userid,or userid<br>2.U_userid~U_agentid<br>3.U_userid~U_agentid/X_XXXid<br>4.U_userid/X_XXXid<br>5./X_XXXid,or /XXXid,or X_XXXid must have a set type prefix, otherwise it will be classified by 'Userid'<br>                                                                                                                                                                            |
### setMsgSendFlagMap
| Category              | Description                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         |
|-----------------------|---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| Parameter format      | HashMap &#60;String,String&#62;<br> Set message sending method<br>key Message sending ID (operator type)<br>Such as:<br> 'Primary ' is the main person in charge, Secondary is  assist processing operator<br>Reader only read permission, Processed refers to the user who have handled the flow<br>Administrator is the process manager ,Creator is the initializer of the process instance<br>value  is sending methods ( multiple methods need to be separated by "&# 124;" )<br>Such as：<br>Todo means work waiting to be processed, To be read refers to the process to be viewed<br>Mail refers to email ,Msg refer to system message<br>SMS refer to short message of mobile phone SMS<br>IM refer to instant messaging,Todo&#124;Read&#124;Mail multivalue |
| Parameter implication | Send message type settings to the next node operator                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                |
| Scope of action       | Instance operation                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  |
| Required              | No                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  |
| Other description     | Not specified, the system use the configuration in the process definition at default                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                |
### setSelectAuthorNameMap
| Category              | Description                                                                                                                                                |
|-----------------------|------------------------------------------------------------------------------------------------------------------------------------------------------------|
| Parameter format      | HashMap &#60;String,String&#62;<br> The key is consistent with the key in 'selectAuthorMap', and value stores the corresponding  name for logging purposes |
| Parameter implication | Name of operator                                                                                                                                           |
| Scope of action       | Instance operation                                                                                                                                         |
| Required              | No                                                                                                                                                         |
### setFlowVarMap
| Category              | Description                                                                                                                                                                              |
|-----------------------|------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| Parameter format      | HashMap &#60;String,String&#62;<br>Key=Process variable name,value=Process variable value                                                                                                |
| Parameter implication | Process variable settings                                                                                                                                                                |
| Scope of action       | Instance operation                                                                                                                                                                       |
| Required              | No                                                                                                                                                                                       |
| Other description     | When performing process variable setting operations, assign values to the process variables. if there is no process variable setting operations, the system uses the default set  values |
### setBussinessClass
| Category              | Description                                   |
|-----------------------|-----------------------------------------------|
| Category              | Description                                   |
| Parameter format      | Character string                              |
| Parameter implication | Business data processing implementation class |
| Scope of action       | Instance operation                            |
| Required              | No                                            |
| Other description     | Interface must be implemented ,IBaseRule      |
### setBussinessClassForOpenByService
| Category              | Description                                                                    |
|-----------------------|--------------------------------------------------------------------------------|
| Parameter format      | Character string                                                               |
| Parameter implication | The implementation class for obtaining business data ,when opening an instance |
| Scope of action       | Open Instance                                                                  |
| Required              | No                                                                             |
### setFunname
| Category              | Description                                                                                                                                                                        |
|-----------------------|------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| Parameter format      | Character string                                                                                                                                                                   |
| Parameter implication | Operation ID                                                                                                                                                                       |
| Scope of action       | Instance operation                                                                                                                                                                 |
| Required              | No                                                                                                                                                                                 |
| Other description     | The operation ID in the configuration file (operation. xml) used for the corresponding process definition <br>![Process operation class diagram](../assets/img/api/setFunname.png) |
### setSubmitflag
| Category              | Description                                                                                                                                                                                                                    |
|-----------------------|--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| Parameter format      | Character string                                                                                                                                                                                                               |
| Parameter implication | Whether to persist the switch when submitting an operation, non persistence is equivalent to preprocessing                                                                                                                     |
| Scope of action       | Instance operation                                                                                                                                                                                                             |
| Required              | No                                                                                                                                                                                                                             |
| Other description     | 0：Confirm persistence  1：Pre processing of front and back interaction<br>If the operating conditions are not met, even if set to 0, the system will not persist, just like the effect of 1,<br>It just consumes more resources |
### setActionname
| Category              | Description                                                                                                                  |
|-----------------------|------------------------------------------------------------------------------------------------------------------------------|
| Parameter format      | Character string                                                                                                             |
| Parameter implication |                                                                                                                              |
| Scope of action       | Instance operation                                                                                                           |
| Required              | No                                                                                                                           |
| Other description     | Used for logging. If the value is not empty, it will be recorded in the log. If it is empty, the button name should be found |
### setClearMemAfterAction
| Category              | Description                                                                                                |
|-----------------------|------------------------------------------------------------------------------------------------------------|
| Parameter format      | boolean                                                                                                    |
| Parameter implication | Clear the corresponding data in the memory or not after the operation is completed                         |
| Scope of action       | Instance operation                                                                                         |
| Required              | No                                                                                                         |
| Other description     | The default is True ,to clear, which is often used for save operations, False is not closing after saving. |
### setRejectOtherBranch
| Category              | Description                |
|-----------------------|----------------------------|
| Parameter format      | Character string           |
| Parameter implication | Need a return label or not |
| Scope of action       | Instance operation         |
| Required              | No                         |
| Other description     | 1Yes,0 No                  |
### setMultiInstanceTransaction
| Category              | Description                                                                                                                                                                                                           |
|-----------------------|-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| Parameter format      | boolean                                                                                                                                                                                                               |
| Parameter implication | Multiple instances of transactions or not                                                                                                                                                                             |
| Scope of action       | Instance operation                                                                                                                                                                                                    |
| Required              | No                                                                                                                                                                                                                    |
| Other description     | Defalt False<br>If set to True, the system will not persist , Persistence will be executed by multiple instance transaction  ,At this time 'getFlowMap()' in 'RunningData' return the persisted SQL and parameter Map |
### setComment
| Category              | Description                                                                                             |
|-----------------------|---------------------------------------------------------------------------------------------------------|
| Parameter format      | Character string                                                                                        |
| Parameter implication | Comment                                                                                                 |
| Scope of action       | Instance operation                                                                                      |
| Required              | No                                                                                                      |
| Other description     | By default, write to the process tracking log, and the write location can also be determined separately |
### setSxComment
| Category              | Description        |
|-----------------------|--------------------|
| Parameter format      | Character string   |
| Parameter implication | Comment2           |
| Scope of action       | Instance operation |
| Required              | No                 |
### setDoNotToFirstNode
| Category              | Description                                                                                                         |
|-----------------------|---------------------------------------------------------------------------------------------------------------------|
| Parameter format      | boolean                                                                                                             |
| Parameter implication | Do not automatically submit to the first node when the process starts. False: Automatically submit to the next node |
| Scope of action       | Instance startup                                                                                                    |
| Required              | No                                                                                                                  |
### setNodeLimitForAction(Parameter contains an array)
| Category              | Description                                                                                                                                                                                                                                                                                |
|-----------------------|--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| Parameter format      | String nodeid, Integer array int[] ns                                                                                                                                                                                                                                                      |
| Parameter implication | Node processing time limit setting,nodeid:Note Id<br>ns: Array order:<br> 0:Processing time limit type(0=Working day,1=Calendar day,2=Hour,3=Minute)1:Processing time<br>2:Warning duration<br>3:The length of the allowed reminders array does not need to be<br>4:Allow settings section |
| Scope of action       | Instance operation                                                                                                                                                                                                                                                                         |
| Required              | No                                                                                                                                                                                                                                                                                         |
| Other description     |                                                                                                                                                                                                                                                                                            |
### setNodeLimitForAction(Argument string)
| Category              | Description                                                                                                                                                                                                                                                                        |
|-----------------------|------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| Parameter format      | String nodeid,Character stringString ns                                                                                                                                                                                                                                            |
| Parameter implication | Node processing time limit setting,nodeid:Node Id<br>ns:String order，Multi values are connected with semicolons in the middle:<br> 0:Processing time limit type(0=Working day,1=Calendar Day,2=Hour,3=Minute)<br>1:Processing time<br>2:Warning duration<br>3:Allow reminder times |
| Scope of action       | Instance operation                                                                                                                                                                                                                                                                 |
| Required              | No                                                                                                                                                                                                                                                                                 |
### setNodeLimitSetMap
| Category              | Description                                                                                                                                                                                                                                                                                          |
|-----------------------|------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| Parameter format      | Map<String,int[]> nodeLimitSetMap                                                                                                                                                                                                                                                                    |
| Parameter implication | Node processing time limit setting,String nodeid:Node Id<br>ns: Array order:<br>0:Processing time limit type(0=Working day,1=Calendar Day,2=Hour,3=Minute)<br>1:Processing time<br>2:Warning duration<br>3:The length of the allowed reminders array does not need to be<br>4:Allow settings section |
| Scope of action       | Instance operation                                                                                                                                                                                                                                                                                   |
| Required              | No                                                                                                                                                                                                                                                                                                   |
### setNextNodeName
| Category              | Description                                                           |
|-----------------------|-----------------------------------------------------------------------|
| Parameter format      | Character string                                                      |
| Parameter implication | When dynamically adding nodes, the node name that needs to be entered |
| Scope of action       | Instance operation                                                    |
| Required              | Required                                                              |
### setNextNodeModuleName
| Category              | Description                                                                 |
|-----------------------|-----------------------------------------------------------------------------|
| Parameter format      | Character string                                                            |
| Parameter implication | When dynamically adding nodes, specify the name of the copied node template |
| Scope of action       | Instance operation                                                          |
| Required              | No                                                                          |
### setNextNodeX
| Category              | Description                                                   |
|-----------------------|---------------------------------------------------------------|
| Parameter format      | Number                                                        |
| Parameter implication | When dynamically adding nodes, the node position coordinate X |
| Scope of action       | Instance operation                                            |
| Required              | No                                                            |
### setNextNodeY
| Category              | Description                                                   |
|-----------------------|---------------------------------------------------------------|
| Parameter format      | Number                                                        |
| Parameter implication | When dynamically adding nodes, the node position coordinate Y |
| Scope of action       | Instance operation                                            |
| Required              | No                                                            |
### setNextNodePosX
| Category              | Description                                                         |
|-----------------------|---------------------------------------------------------------------|
| Parameter format      | Number                                                              |
| Parameter implication | The X displacement of the dynamic node relative to the current node |
| Scope of action       | Instance operation                                                  |
| Required              | No                                                                  |
### setNextNodePosY
| Category   | Description              |
|------|----------------|
| Parameter format | Number             |
| Parameter implication | The Y displacement of the dynamic node relative to the current node ||
| Scope of action | Instance operation           |
| Required | No           |

## WorkResultBean  (Return result parameter description)
### getResult
| Category                 | Description                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 |
|--------------------------|-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| Return type              | int                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         |
| Scope of action          | Open instance,Create an instance,Operation execution                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        |
| Return value description | Get return code<br>0:Success 1：Unable to obtain the next node, 2：Prompt to select nodes, 3：Prompt that the operator is empty,<br>4：Prompt for selecting operator, 5：End of process, 6：Sub process not ended,<br>7：Process pause,8：Process abnormal termination ,9：Other error prompts,<br>14:The sub process has already been initiated, 15：Need to select a sub process, 16:Submitted by the secondary operator,<br>17:Does not meet the leaving rule for merged nodes<br>22:Exception in entering rule invocation,23：Leaving rule invocation  Error,<br>24:This node type does not support corresponding operations - ‘Parallel submit’ or ‘Add operator’,<br>25：Enable rule invocation error,  26：Node activation failed |
### getBackMsg
| Category                 | Description                                       |
|--------------------------|---------------------------------------------------|
| Return type              | String                                            |
| Scope of action          | Open instance,Create instance,Operation execution |
| Return value description | Obtain returned prompt information                |
### getActiveTrackInfos
| Category                 | Description                                                                                               |
|--------------------------|-----------------------------------------------------------------------------------------------------------|
| Return type              | List &#60;IXmlTrack&#62;                                                                                  |
| Scope of action          | Open instance                                                                                             |
| Return value description | Obtain a list of active node information when there are multiple optional paths after opening an instance |
### getCurTrackInfo
| Category                 | Description                                                |
|--------------------------|------------------------------------------------------------|
| Return type              | DBTrack                                                    |
| Scope of action          | Open instance,Create instance,Instance operation execution |
| Return value description | Obtain the current path information object                 |
### getNextTrackInfos
| Category                 | Description                              |
|--------------------------|------------------------------------------|
| Return type              | DBTrack                                  |
| Scope of action          | Instance operation  execution            |
| Return value description | Obtain the list of next path information |
### getCurNode
| Category                 | Description                                              |
|--------------------------|----------------------------------------------------------|
| Return type              | IXmlNode                                                 |
| Scope of action          | Open instance,Create instance,Operation execution        |
| Return value description | Obtain the current node configuration information object |
### getCurUser
| Category                 | Description                                       |
|--------------------------|---------------------------------------------------|
| Return type              | IRunUser                                          |
| Scope of action          | Open instance,Create instance,Operation execution |
| Return value description | Current executor information object               |
### getWork
| Category                 | Description                                       |
|--------------------------|---------------------------------------------------|
| Return type              | DBWork                                            |
| Scope of action          | Open instance,Create instance,Operation execution |
| Return value description | Obtain the current instance information object    |
### getFlowInfo
| Category                 | Description                                                                |
|--------------------------|----------------------------------------------------------------------------|
| Return type              | IXmlFlowInfo                                                               |
| Scope of action          | Open instance,Create instance,Operation execution                          |
| Return value description | Obtain the process definition object corresponding to the current instance |
### getFlowVarMap
| Category                 | Description                                |
|--------------------------|--------------------------------------------|
| Return type              | HashMap &#60;String,String&#62;            |
| Scope of action          | Open instance,Operation execution          |
| Return value description | Process variables map of  current instance |
### getHashMap
| Category                 | Description                                                       |
|--------------------------|-------------------------------------------------------------------|
| Return type              | HashMap &#60;String,Object&#62;                                   |
| Scope of action          | Operation execution                                               |
| Return value description | The parameters passed in before the current operation is executed |
### getWorkParaBean
| Category                 | Description                                                                                   |
|--------------------------|-----------------------------------------------------------------------------------------------|
| Return type              | WorkParaBean                                                                                  |
| Scope of action          | Open instance,Create instance,Operation execution                                             |
| Return value description | Open instance,Create instance,When the operation is executed, a parameter object is passed in |
### getRunningdata
| Category                 | Description                                                               |
|--------------------------|---------------------------------------------------------------------------|
| Return type              | RunningData                                                               |
| Scope of action          | Open instance,Create instance,Operation execution                         |
| Return value description | The runtime parameter object, from which other parameters can be obtained |
### getNextNodes
| Category                 | Description                                                                                                                                                                                                           |
|--------------------------|-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| Return type              | INextNode                                                                                                                                                                                                             |
| Scope of action          | Operation execution                                                                                                                                                                                                   |
| Return value description | During the interaction process of operation execution, obtain the next node list information, and display.<br>For example, whether a node needs to select operator, and whether the node is single or multiple choice |
### getNextNodeids
| Category                 | Description                                                                                                                              |
|--------------------------|------------------------------------------------------------------------------------------------------------------------------------------|
| Return type              | String                                                                                                                                   |
| Scope of action          | Operation execution                                                                                                                      |
| Return value description | After obtaining the nodes and operators, pass in the next node id  list,<br>Obtain node information when application prompt confirmation |
### getSelectAuthorMap
| Category                 | Description                                                                                                                                                                                                                                                            |
|--------------------------|------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| Return type              | HashMap &#60;String,String&#62;                                                                                                                                                                                                                                        |
| Scope of action          | Operation execution                                                                                                                                                                                                                                                    |
| Return value description | The operator information obtained by the next node refers to the 'setSelectAuthorMap',<br>The difference between this map and the map set in the 'setSelectAuthorMap' method is that the returned map has been modified by the engine,<br>There may be inconsistencies |
### getTmpAuthorMap
| Category                 | Description                                                                            |
|--------------------------|----------------------------------------------------------------------------------------|
| Return type              | HashMap &#60;String,String&#62;                                                        |
| Scope of action          | Operation execution                                                                    |
| Return value description | The temporary storage list of the next node operator, not the final confirmed operator |
### getInitAuthorMap
| Category                 | Description                                                              |
|--------------------------|--------------------------------------------------------------------------|
| Return type              | HashMap &#60;String,String&#62;                                          |
| Scope of action          | Operation execution                                                      |
| Return value description | The operator list defined by the next node ,it is not the final operator |
### getSubflowWorks
| Category                 | Description                                                                                          |
|--------------------------|------------------------------------------------------------------------------------------------------|
| Return type              | List &#60;DBRelationSubflow&#62;                                                                     |
| Scope of action          | start sub_process operations                                                                         |
| Return value description | The started subprocess instance corresponding to the current instance and its associated information |
### getSubflows
| Category                 | Description                                                                        |
|--------------------------|------------------------------------------------------------------------------------|
| Return type              | List &#60;IXmlSubFlowInfo&#62;                                                     |
| Scope of action          | start sub_process operations                                                       |
| Return value description | Obtain the definition information of the current sub processes that can be started |
### getSubflowWorkResult
| Category                 | Description                                         |
|--------------------------|-----------------------------------------------------|
| Return type              | IWorkResult                                         |
| Scope of action          | start sub_process operations                        |
| Return value description | Return object after subprocess startup is completed |
### getCurNodeAuthor
| Category                 | Description                                                                                                                                                                                                                                                                                                                                                                                                                |
|--------------------------|----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| Return type              | String                                                                                                                                                                                                                                                                                                                                                                                                                     |
| Scope of action          | Open instance                                                                                                                                                                                                                                                                                                                                                                                                              |
| Return value description | The current format of personnel used for application display includes the following situations：<br>1.U_userid   or userid<br>2.U_userid~U_agentid<br>3.U_userid~U_agentid/X_XXXid<br>4.U_userid/D_deptid<br>5./X_XXXid,or /XXXid ,or X_XXXid,Must have a set type prefix, otherwise it will be classified by userid<br>6.[U_userid~U_agentid]/D_deptid<br>7.[U_userid]/Claim processing records recorded in 'D_deptid xml' |
### getFlowStatusName
| Category                 | Description                                                               |
|--------------------------|---------------------------------------------------------------------------|
| Return type              | String                                                                    |
| Scope of action          | Open instance                                                             |
| Return value description | Convert the process status to display in the current environment language |
### getNextSelectTypeByNodeid
| Category                 | Description                                                                                                                                                      |
|--------------------------|------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| Return type              | int                                                                                                                                                              |
| Scope of action          | Operation execution                                                                                                                                              |
| Return value description | The method of selecting operator for this node based on the nodeid is: 1. Single selection, 2. Multiple selection (funname also participates in the calculation) |
### getDataidByFormid
| Category                 | Description                                                                    |
|--------------------------|--------------------------------------------------------------------------------|
| Return type              | String                                                                         |
| Scope of action          | Open instance                                                                  |
| Return value description | Obtain the business data ID based on the formid in the 'TWR_HZ_Instance' table |
### getRelationsByWorkid
| Category                 | Description                               |
|--------------------------|-------------------------------------------|
| Return type              | List &#60;DBRelation&#62;                 |
| Scope of action          | Open instance                             |
| Return value description | Obtain association data based on 'Workid' |
### getWorkid
| Category                 | Description                                       |
|--------------------------|---------------------------------------------------|
| Return type              | String                                            |
| Scope of action          | Open instance,Create instance,Operation execution |
| Return value description | Returns the current instance ID                   |
### isNowUserLock
| Category                 | Description                  |
|--------------------------|------------------------------|
| Return type              | Boolean                      |
| Scope of action          | Open instance                |
| Return value description | Is the current person locked |
### getLockUser
| Category                 | Description                 |
|--------------------------|-----------------------------|
| Return type              | String                      |
| Scope of action          | Open instance               |
| Return value description | Return to the locked person |
### getOpenInstance
| Category                 | Description          |
|--------------------------|----------------------|
| Return type              | OpenInstance         |
| Scope of action          | Open instance        |
| Return value description | Instance open object |
### getCreateInstance
| Category                 | Description            |
|--------------------------|------------------------|
| Return type              | CreateInstance         |
| Scope of action          | Create instance        |
| Return value description | Create instance object |
### getCurButton
| Category                 | Description                                                                |
|--------------------------|----------------------------------------------------------------------------|
| Return type              | List &#60;IXmlButton&#62;                                                  |
| Scope of action          | Open instance,Create instance                                              |
| Return value description | Obtain the list of operations that the current operator has permissions on |

