## 组织机构API说明
>该文档主要是对工作流引擎提供的API接口进行简要说明

### 核心入口类
源代码位置：horizon-workflow-extend开头的jar包内com.horizon.third路径下。
组织机构数据查询和组织机构缓存数据清理接口获取的工厂类: com.horizon.third.OrgDataFactory
组织机构数据查询接口类：com.horizon.third.service.api.OrgDataQueryApi
组织机构缓存数据清理接口类: com.horizon.third.service.api.OrgDataCacheApi
组织机构数据查询返回类：com.horizon.third.entity.OrgData
组织机构用户数据查询返回类：com.horizon.third.entity.User
组织机构经过处理的树形结构数据com.horizon.core.tree.TreeModel<T>
### 组织机构数据查询调用说明
查询组织机构数据，主要包括部门、群组、岗位、角色、用户的数据。
以上所有操作都是通过OrgDataQueryApi类操作。总体调用代码结构如下：
```
//获取操作实现类
OrgDataQueryApi orgDataQuery = OrgDataFactory.getOrgDataQueryManager();
```

### 组织机构缓存数据清理调用说明
清理放入缓存中的组织机构数据，主要包括部门、群组、岗位、角色、用户的缓存数据
以上所有操作都是通过OrgDataCacheApi类实现。总体调用代码结构如下：
```
//获取操作实现类
OrgDataCacheApi orgDataCache= OrgDataFactory.getOrgDataCacheManager();
```
### 结果集说明
**OrgData结果集说明**

| 属性       | 类型         | 是否必须 | 描述                                     |
|----------|------------|------|----------------------------------------|
| id       | String     | 是    | 组织机构id                                 |
| name     | String     | 是    | 组织机构名称                                 |
| type     | String     | 是    | 组织机构类型，限定值<br>  D：部门; G：群组; P：岗位; R：角色 |
| parentId | String     | 否    | 父级ID                                   |
| deptId   | String     | 否    | 所属部门ID                                 |
| users    | List<User> | 否    | 该组织机构下存在的用户                            |
**User结果集说明**

| 属性         | 类型           | 是否必须 | 描述                             |
|------------|--------------|------|--------------------------------|
| id         | String       | 是    | 用户id                           |
| userName   | String       | 是    | 用户名                            |
| loginName  | String       | 是    | 登录名                            |
| password   | List         | 是    | 登录密码                           |
| department | List<String> | 否    | 用户所在的部门ID集合，包括兼职部门，其中第一个值为直接部门 |

## OrgDataQueryApi方法调用说明
### findUserByOrgId
**服务描述**
<table>
    <tr>
        <td>方法名称</td>
        <td>findUserByOrgId</td>
        <td>中文名称</td>
        <td></td>
    </tr>
    <tr>
        <td>功能描述</td>
        <td colspan="3">根据组织机构的类型和ID查找用户</td>
    </tr>
    <tr>
        <td colspan="4" align="center">接口信息</td>
    </tr>
    <tr>
        <td>接口类型</td>
        <td>api</td>
        <td>交互频度</td>
        <td>非周期</td>
    </tr>
    <tr>
        <td>接口协议</td>
        <td>直接方法调用</td>
        <td>响应时间</td>
        <td></td>
    </tr>
</table>

**参数说明**

| 参数名称  | 类型     | 是否必填 | 描述                                    |
|-------|--------|------|---------------------------------------|
| type  | String | √    | type：组织机构类，限定值<br>D：部门;G：群组;P：岗位;R：角色 |
| orgId | String | √    | 组织机构ID                                |
**调用示例**
```
OrgDataQueryApi orgDataQuery = OrgDataFactory.getOrgDataQueryManager();
List<User> users = orgDataQuery.findUserByOrgId(“D”, ”HZ28e7f51e816d28011e816da7f1001f”);
```
### findUserLikeParam
**服务描述**
 <table>
    <tr>
        <td>方法名称</td>
        <td>findUserLikeParam</td>
        <td>中文名称</td>
        <td></td>
    </tr>
    <tr>
        <td>功能描述</td>
        <td colspan="3">根据传参值param模糊检索用户,最多返回searchMax个用户</td>
    </tr>
    <tr>
        <td colspan="4" align="center">接口信息</td>
    </tr>
    <tr>
        <td>接口类型</td>
        <td>api</td>
        <td>交互频度</td>
        <td>非周期</td>
    </tr>
    <tr>
        <td>接口协议</td>
        <td>直接方法调用</td>
        <td>响应时间</td>
        <td></td>
    </tr>
</table>

**参数说明**

| 参数名称      | 类型     | 是否必填 | 描述                 |
|-----------|--------|------|--------------------|
| param     | String | √    | 用户名或者登录名模糊搜索关键字符串  |
| searchMax | String | √    | 当有多个匹配用户时最多返回多少个用户 |
**调用示例**
```
OrgDataQueryApi orgDataQuery = OrgDataFactory.getOrgDataQueryManager();
List<User> users = orgDataQuery.findUserLikeParam(5, ”user01”);
```
### getUserById
**服务描述**
 <table>
    <tr>
        <td>方法名称</td>
        <td>getUserById</td>                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               
        <td>中文名称</td>
        <td></td>
    </tr>
    <tr>
        <td>功能描述</td>
        <td colspan="3">根据用户ID检索用户</td>
    </tr>
    <tr>
        <td colspan="4" align="center">接口信息</td>
    </tr>
    <tr>
        <td>接口类型</td>
        <td>api</td>
        <td>交互频度</td>
        <td>非周期</td>
    </tr>
    <tr>
        <td>接口协议</td>
        <td>直接方法调用</td>
        <td>响应时间</td>
        <td></td>
    </tr>
</table>

**参数说明**

| 参数名称 | 类型     | 是否必填 | 描述   |
|------|--------|------|------|
| id   | String | √    | 用户ID |
**调用示例**
```
OrgDataQueryApi orgDataQuery = OrgDataFactory.getOrgDataQueryManager();
User user = orgDataQuery.getUserById(“admin”);
```
### getUserByLoginName
**服务描述**
 <table>
    <tr>
        <td>方法名称</td>
        <td>getUserByLoginName</td>
        <td>中文名称</td>
        <td></td>
    </tr>
    <tr>
        <td>功能描述</td>
        <td colspan="3">根据登录名检索用户</td>
    </tr>
    <tr>
        <td colspan="4" align="center">接口信息</td>
    </tr>
    <tr>
        <td>接口类型</td>
        <td>api</td>
        <td>交互频度</td>
        <td>非周期</td>
    </tr>
    <tr>
        <td>接口协议</td>
        <td>直接方法调用</td>
        <td>响应时间</td>
        <td></td>
    </tr>
</table>

**参数说明**

| 参数名称      | 类型     | 是否必填 | 描述    |
|-----------|--------|------|-------|
| loginName | String | √    | 用户登录名 |
**调用示例**
```
OrgDataQueryApi orgDataQuery = OrgDataFactory.getOrgDataQueryManager();
User user = orgDataQuery.getUserByLoginName (“admin”);
```
### findAllUser
**服务描述**
 <table>
    <tr>
        <td>方法名称</td>
        <td>findAllUser</td>
        <td>中文名称</td>
        <td></td>
    </tr>
    <tr>
        <td>功能描述</td>
        <td colspan="3">查询所有用户</td>
    </tr>
    <tr>
        <td colspan="4" align="center">接口信息</td>
    </tr>
    <tr>
        <td>接口类型</td>
        <td>api</td>
        <td>交互频度</td>
        <td>非周期</td>
    </tr>
    <tr>
        <td>接口协议</td>
        <td>直接方法调用</td>
        <td>响应时间</td>
        <td></td>
    </tr>
</table>

**调用示例**
```
OrgDataQueryApi orgDataQuery = OrgDataFactory.getOrgDataQueryManager();
List<User> users = orgDataQuery.findAllUser();
```
### findOrgForUserByUserId
**服务描述**
 <table>
    <tr>
        <td>方法名称</td>
        <td>findOrgForUserByUserId</td>
        <td>中文名称</td>
        <td></td>
    </tr>
    <tr>
        <td>功能描述</td>
        <td colspan="3">根据用户id查找用户所属的组织机构</td>
    </tr>
    <tr>
        <td colspan="4" align="center">接口信息</td>
    </tr>
    <tr>
        <td>接口类型</td>
        <td>api</td>
        <td>交互频度</td>
        <td>非周期</td>
    </tr>
    <tr>
        <td>接口协议</td>
        <td>直接方法调用</td>
        <td>响应时间</td>
        <td></td>
    </tr>
</table>

**参数说明**

| 参数名称   | 类型     | 是否必填 | 描述   |
|--------|--------|------|------|
| userId | String | √    | 用户ID |
**调用示例**
```
OrgDataQueryApi orgDataQuery = OrgDataFactory.getOrgDataQueryManager();
List<OrgData> orgDatas = orgDataQuery.findOrgForUserByUserId(“admin”);
```
### findOrgForUserByUserId
**服务描述**
 <table>
    <tr>
        <td>方法名称</td>
        <td>findOrgForUserByUserId</td>
        <td>中文名称</td>
        <td></td>
    </tr>
    <tr>
        <td>功能描述</td>
        <td colspan="3">根据用户ID和组织机构类型查找用户所在的组织机构</td>
    </tr>
    <tr>
        <td colspan="4" align="center">接口信息</td>
    </tr>
    <tr>
        <td>接口类型</td>
        <td>api</td>
        <td>交互频度</td>
        <td>非周期</td>
    </tr>
    <tr>
        <td>接口协议</td>
        <td>直接方法调用</td>
        <td>响应时间</td>
        <td></td>
    </tr>
</table>

**参数说明**

| 参数名称   | 类型     | 是否必填 | 描述                                    |
|--------|--------|------|---------------------------------------|
| type   | String | √    | type：组织机构类，限定值<br>D：部门;G：群组;P：岗位;R：角色 |
| userId | String | √    | 用户Id                                  |
**调用示例**
```
OrgDataQueryApi orgDataQuery = OrgDataFactory.getOrgDataQueryManager();
List<OrgData> orgDatas = orgDataQuery.findOrgForUserByUserId (“D”, “admin”);
```
### getOrgDataById
**服务描述**
 <table>
    <tr>
        <td>方法名称</td>
        <td>getOrgDataById</td>
        <td>中文名称</td>
        <td></td>
    </tr>
    <tr>
        <td>功能描述</td>
        <td colspan="3">根据组织机构的类型和ID查找对应给组织机构数据</td>
    </tr>
    <tr>
        <td colspan="4" align="center">接口信息</td>
    </tr>
    <tr>
        <td>接口类型</td>
        <td>api</td>
        <td>交互频度</td>
        <td>非周期</td>
    </tr>
    <tr>
        <td>接口协议</td>
        <td>直接方法调用</td>
        <td>响应时间</td>
        <td></td>
    </tr>
</table>

**参数说明**

| 参数名称  | 类型     | 是否必填 | 描述                                    |
|-------|--------|------|---------------------------------------|
| type  | String | √    | type：组织机构类，限定值<br>D：部门;G：群组;P：岗位;R：角色 |
| orgId | String | √    | 组织机构ID                                |
**调用示例**
```
OrgDataQueryApi orgDataQuery = OrgDataFactory.getOrgDataQueryManager();
OrgData orgData = orgDataQuery.getOrgDataById(“D”, “admin”);
```
### findChildrenData
**服务描述**
 <table>
    <tr>
        <td>方法名称</td>
        <td>findChildrenData</td>
        <td>中文名称</td>
        <td></td>
    </tr>
    <tr>
        <td>功能描述</td>
        <td colspan="3">根据组织机构的类型和ID查找对应的组织机构下的子级，查询所有子级的子级</td>
    </tr>
    <tr>
        <td colspan="4" align="center">接口信息</td>
    </tr>
    <tr>
        <td>接口类型</td>
        <td>api</td>
        <td>交互频度</td>
        <td>非周期</td>
    </tr>
    <tr>
        <td>接口协议</td>
        <td>直接方法调用</td>
        <td>响应时间</td>
        <td></td>
    </tr>
    <tr>
        <td>备注</td>
        <td colspan="3">只能查询直接下级</td>
    </tr>
</table>

**参数说明**

| 参数名称  | 类型     | 是否必填 | 描述                                    |
|-------|--------|------|---------------------------------------|
| type  | String | √    | type：组织机构类，限定值<br>D：部门;G：群组;P：岗位;R：角色 |
| orgId | String | √    | 组织机构ID                                |
**调用示例**
```
OrgDataQueryApi orgDataQuery = OrgDataFactory.getOrgDataQueryManager();
List<OrgData> orgDatas = orgDataQuery.findChildrenData (“D”, “HZ28e7f51e816d28011e816da7f1001f”);
```
### findChildrenData
**服务描述**
 <table>
    <tr>
        <td>方法名称</td>
        <td>findChildrenData</td>
        <td>中文名称</td>
        <td></td>
    </tr>
    <tr>
        <td>功能描述</td>
        <td colspan="3">根据组织机构的类型和ID查找对应的组织机构下的子级，查询所有子级的子级</td>
    </tr>
    <tr>
        <td colspan="4" align="center">接口信息</td>
    </tr>
    <tr>
        <td>接口类型</td>
        <td>api</td>
        <td>交互频度</td>
        <td>非周期</td>
    </tr>
    <tr>
        <td>接口协议</td>
        <td>直接方法调用</td>
        <td>响应时间</td>
        <td></td>
    </tr>
    <tr>
        <td>备注</td>
        <td colspan="3">可以选择查询所有子级</td>
    </tr>
</table>

**参数说明**

| 参数名称       | 类型      | 是否必填 | 描述                                    |
|------------|---------|------|---------------------------------------|
| type       | String  | √    | type：组织机构类，限定值<br>D：部门;G：群组;P：岗位;R：角色 |
| orgId      | String  | √    | 组织机构ID                                |
| isAllChild | boolean | √    | 是否查询所有子级                              |
**调用示例**
```
OrgDataQueryApi orgDataQuery = OrgDataFactory.getOrgDataQueryManager();
List<User> users = orgDataQuery.findUserByOrgId(“D”, ”HZ28e7f51e816d28011e816da7f1001f”, true);
```
### getRootNode
**服务描述**
 <table>
    <tr>
        <td>方法名称</td>
        <td>getRootNode</td>
        <td>中文名称</td>
        <td></td>
    </tr>
    <tr>
        <td>功能描述</td>
        <td colspan="3">根据组织机构的类型查询该组织机构的根级</td>
    </tr>
    <tr>
        <td colspan="4" align="center">接口信息</td>
    </tr>
    <tr>
        <td>接口类型</td>
        <td>api</td>
        <td>交互频度</td>
        <td>非周期</td>
    </tr>
    <tr>
        <td>接口协议</td>
        <td>直接方法调用</td>
        <td>响应时间</td>
        <td></td>
    </tr>
</table>

**参数说明**

| 参数名称  | 类型     | 是否必填 | 描述                                    |
|-------|--------|------|---------------------------------------|
| type  | String | √    | type：组织机构类，限定值<br>D：部门;G：群组;P：岗位;R：角色 |
**调用示例**
```
OrgDataQueryApi orgDataQuery = OrgDataFactory.getOrgDataQueryManager();
OrgData orgData = orgDataQuery.getRootNode(“D”);
```
### findParentToRoot
**服务描述**
 <table>
    <tr>
        <td>方法名称</td>
        <td>findParentToRoot</td>
        <td>中文名称</td>
        <td></td>
    </tr>
    <tr>
        <td>功能描述</td>
        <td colspan="3">根据组织机构的类型和ID查找所有的上级</td>
    </tr>
    <tr>
        <td colspan="4" align="center">接口信息</td>
    </tr>
    <tr>
        <td>接口类型</td>
        <td>api</td>
        <td>交互频度</td>
        <td>非周期</td>
    </tr>
    <tr>
        <td>接口协议</td>
        <td>直接方法调用</td>
        <td>响应时间</td>
        <td></td>
    </tr>
</table>

**参数说明**

| 参数名称  | 类型     | 是否必填 | 描述                                    |
|-------|--------|------|---------------------------------------|
| type  | String | √    | type：组织机构类，限定值<br>D：部门;G：群组;P：岗位;R：角色 |
| orgId | String | √    | 组织机构ID                                |
**调用示例**
```
OrgDataQueryApi orgDataQuery = OrgDataFactory.getOrgDataQueryManager();
List<OrgData> orgDatas = orgDataQuery.findParentToRoot(“D”, “HZ28e7f51e816d28011e816da7f1001f”);
```
### findOrgData
**服务描述**
 <table>
    <tr>
        <td>方法名称</td>
        <td>findOrgData</td>
        <td>中文名称</td>
        <td></td>
    </tr>
    <tr>
        <td>功能描述</td>
        <td colspan="3">据组织机构的类型查找该类型下的所有组织机构数据</td>
    </tr>
    <tr>
        <td colspan="4" align="center">接口信息</td>
    </tr>
    <tr>
        <td>接口类型</td>
        <td>api</td>
        <td>交互频度</td>
        <td>非周期</td>
    </tr>
    <tr>
        <td>接口协议</td>
        <td>直接方法调用</td>
        <td>响应时间</td>
        <td></td>
    </tr>
</table>

**参数说明**

| 参数名称  | 类型     | 是否必填 | 描述                                    |
|-------|--------|------|---------------------------------------|
| type  | String | √    | type：组织机构类，限定值<br>D：部门;G：群组;P：岗位;R：角色 |
**调用示例**
```
OrgDataQueryApi orgDataQuery = OrgDataFactory.getOrgDataQueryManager();
List<OrgData> orgDatas = orgDataQuery.findOrgData(“D”);
```
### findOrgData
**服务描述**
 <table>
    <tr>
        <td>方法名称</td>
        <td>findOrgData</td>
        <td>中文名称</td>
        <td></td>
    </tr>
    <tr>
        <td>功能描述</td>
        <td colspan="3">获取所有类型的组织机构数据,包括D：部门;G：群组;P：岗位;R：角色</td>
    </tr>
    <tr>
        <td colspan="4" align="center">接口信息</td>
    </tr>
    <tr>
        <td>接口类型</td>
        <td>api</td>
        <td>交互频度</td>
        <td>非周期</td>
    </tr>
    <tr>
        <td>接口协议</td>
        <td>直接方法调用</td>
        <td>响应时间</td>
        <td></td>
    </tr>
</table>

**调用示例**
```
OrgDataQueryApi orgDataQuery = OrgDataFactory.getOrgDataQueryManager();
List<OrgData> orgDatas = orgDataQuery.findOrgData();
```
### getOrgTree
**服务描述**
<table>
    <tr>
        <td>方法名称</td>
        <td>getOrgTree</td>
        <td>中文名称</td>
        <td></td>
    </tr>
    <tr>
        <td>功能描述</td>
        <td colspan="3">据组织机构的类型查找该类型下的所有组织机构数据</td>
    </tr>
    <tr>
        <td colspan="4" align="center">接口信息</td>
    </tr>
    <tr>
        <td>接口类型</td>
        <td>api</td>
        <td>交互频度</td>
        <td>非周期</td>
    </tr>
    <tr>
        <td>接口协议</td>
        <td>直接方法调用</td>
        <td>响应时间</td>
        <td></td>
    </tr>
</table>

**参数说明**

| 参数名称  | 类型     | 是否必填 | 描述                                    |
|-------|--------|------|---------------------------------------|
| type  | String | √    | type：组织机构类，限定值<br>D：部门;G：群组;P：岗位;R：角色 |

**调用示例**
```
OrgDataQueryApi orgDataQuery = OrgDataFactory.getOrgDataQueryManager();
TreeModel<OrgData> tree = orgDataQuery.getOrgTree(“D”);
```

## OrgDataCacheApi方法调用说明

### clearAllOrgCache
**服务描述**
<table>
    <tr>
        <td>方法名称</td>
        <td>clearAllOrgCache</td>
        <td>中文名称</td>
        <td></td>
    </tr>
    <tr>
        <td>功能描述</td>
        <td colspan="3">清除所有类型的组织机构缓存，包括D：部门;G：群组;P：岗位;R：角色</td>
    </tr>
    <tr>
        <td colspan="4" align="center">接口信息</td>
    </tr>
    <tr>
        <td>接口类型</td>
        <td>api</td>
        <td>交互频度</td>
        <td>非周期</td>
    </tr>
    <tr>
        <td>接口协议</td>
        <td>直接方法调用</td>
        <td>响应时间</td>
        <td></td>
    </tr>
</table>

**调用示例**
```
OrgDataQueryApi orgDataCache = OrgDataFactory.getOrgDataCacheManager();
orgDataCache.clearAllOrgCache();
```
### clearOrgCache
**服务描述**
<table>
    <tr>
        <td>方法名称</td>
        <td>clearOrgCache</td>
        <td>中文名称</td>
        <td></td>
    </tr>
    <tr>
        <td>功能描述</td>
        <td colspan="3">清除所有指定类型的组织机构缓存</td>
    </tr>
    <tr>
        <td colspan="4" align="center">接口信息</td>
    </tr>
    <tr>
        <td>接口类型</td>
        <td>api</td>
        <td>交互频度</td>
        <td>非周期</td>
    </tr>
    <tr>
        <td>接口协议</td>
        <td>直接方法调用</td>
        <td>响应时间</td>
        <td></td>
    </tr>
</table>

**参数说明**

| 参数名称  | 类型     | 是否必填 | 描述                                    |
|-------|--------|------|---------------------------------------|
| type  | String | √    | type：组织机构类，限定值<br>D：部门;G：群组;P：岗位;R：角色 |
**调用示例**
```
OrgDataQueryApi orgDataCache = OrgDataFactory.getOrgDataCacheManager();
orgDataCache. clearOrgCache(“D”);
```
### clearDeptCache
**服务描述**
<table>
    <tr>
        <td>方法名称</td>
        <td>clearDeptCache</td>
        <td>中文名称</td>
        <td></td>
    </tr>
    <tr>
        <td>功能描述</td>
        <td colspan="3">清除所有的部门缓存</td>
    </tr>
    <tr>
        <td colspan="4" align="center">接口信息</td>
    </tr>
    <tr>
        <td>接口类型</td>
        <td>api</td>
        <td>交互频度</td>
        <td>非周期</td>
    </tr>
    <tr>
        <td>接口协议</td>
        <td>直接方法调用</td>
        <td>响应时间</td>
        <td></td>
    </tr>
</table>

**调用示例**
```
OrgDataQueryApi orgDataCache = OrgDataFactory.getOrgDataCacheManager();
orgDataCache. clearDeptCache()
```
### clearGroupCache
**服务描述**
<table>
    <tr>
        <td>方法名称</td>
        <td>clearGroupCache</td>
        <td>中文名称</td>
        <td></td>
    </tr>
    <tr>
        <td>功能描述</td>
        <td colspan="3">清除所有的群组缓存</td>
    </tr>
    <tr>
        <td colspan="4" align="center">接口信息</td>
    </tr>
    <tr>
        <td>接口类型</td>
        <td>api</td>
        <td>交互频度</td>
        <td>非周期</td>
    </tr>
    <tr>
        <td>接口协议</td>
        <td>直接方法调用</td>
        <td>响应时间</td>
        <td></td>
    </tr>
</table>

**调用示例**
```
OrgDataQueryApi orgDataCache = OrgDataFactory.getOrgDataCacheManager();
orgDataCache.clearGroupCache();
```
### clearPositionCache
**服务描述**
<table>
    <tr>
        <td>方法名称</td>
        <td>clearPositionCache</td>
        <td>中文名称</td>
        <td></td>
    </tr>
    <tr>
        <td>功能描述</td>
        <td colspan="3">清除所有的岗位缓存</td>
    </tr>
    <tr>
        <td colspan="4" align="center">接口信息</td>
    </tr>
    <tr>
        <td>接口类型</td>
        <td>api</td>
        <td>交互频度</td>
        <td>非周期</td>
    </tr>
    <tr>
        <td>接口协议</td>
        <td>直接方法调用</td>
        <td>响应时间</td>
        <td></td>
    </tr>
</table>

**调用示例**
```
OrgDataQueryApi orgDataCache = OrgDataFactory.getOrgDataCacheManager();
orgDataCache.clearGroupCache();
```
### clearRoleCache
**服务描述**
<table>
    <tr>
        <td>方法名称</td>
        <td>clearRoleCache</td>
        <td>中文名称</td>
        <td></td>
    </tr>
    <tr>
        <td>功能描述</td>
        <td colspan="3">清除所有的角色缓存</td>
    </tr>
    <tr>
        <td colspan="4" align="center">接口信息</td>
    </tr>
    <tr>
        <td>接口类型</td>
        <td>api</td>
        <td>交互频度</td>
        <td>非周期</td>
    </tr>
    <tr>
        <td>接口协议</td>
        <td>直接方法调用</td>
        <td>响应时间</td>
        <td></td>
    </tr>
</table>

**调用示例**
```
OrgDataQueryApi orgDataCache = OrgDataFactory.getOrgDataCacheManager();
orgDataCache.clearRoleCache();
```
### clearUserCache	
**服务描述**
<table>
    <tr>
        <td>方法名称</td>
        <td>clearUserCache</td>
        <td>中文名称</td>
        <td></td>
    </tr>
    <tr>
        <td>功能描述</td>
        <td colspan="3">清除所有的用户缓存</td>
    </tr>
    <tr>
        <td colspan="4" align="center">接口信息</td>
    </tr>
    <tr>
        <td>接口类型</td>
        <td>api</td>
        <td>交互频度</td>
        <td>非周期</td>
    </tr>
    <tr>
        <td>接口协议</td>
        <td>直接方法调用</td>
        <td>响应时间</td>
        <td></td>
    </tr>
</table>

**调用示例**
```
OrgDataQueryApi orgDataCache = OrgDataFactory.getOrgDataCacheManager();
orgDataCache.clearUserCache();
```